export const createSVG = () => {
  const svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
  svg.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
  svg.setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns', 'http://www.w3.org/2000/svg');
  return svg;
};

export const getPathDByData = (dData) => {
  return dData.reduce((previousValue, el) => {
    switch (el.type) {
      case 'M':
        return (previousValue += `M${el.x} ${el.y} `);
      case 'L':
        return (previousValue += `L${el.x} ${el.y} `);
      case 'Q':
        return (previousValue += `Q${el.x} ${el.y} ${el.x1} ${el.y1}`);
      case 'Z':
        return (previousValue += `Z`);
    }
  }, '');
};

export const getPathByCell = (cell, strokeWidth) => {
  if (cell.svg.type === 'path') {
    const d = getPathDByData(cell.svg.dData);
    return `<path fill="white" stroke="black" stroke-width="${strokeWidth}" d="${d}"></path>`;
  }
  if (cell.svg.type === 'circle') {
    return `<circle fill="white" stroke="black" stroke-width="${strokeWidth}" cx="${cell.svg.cx}" cy="${cell.svg.cy}" r="${cell.svg.r}"></circle>`;
  }
};

export const getImageByUrl = (url) => {
  return new Promise((resolve) => {
    const img = new Image();
    img.src = url;
    img.onload = () => {
      resolve(img);
    };
  });
};

export const getFlagsArray = (prefix) => {
  return new Array(6).fill(null).map((_, index) => ({
    name: `${prefix}-flag-stars-${index}`,
    starsN: index,
    prefix,
    index,
  }));
};

export const getSvgPathBorder = (cell, svgWidth, svgHeight, PATH_STROKE_WIDTH, PATH_STROKE_WIDTH_FOR_MASK) => {
  const svgPathMaskContainer = createSVG();
  svgPathMaskContainer.setAttribute('viewBox', `0 0 ${svgWidth} ${svgHeight}`);

  if (cell.svg.type === 'path') {
    const d = getPathDByData(cell.svg.dData);
    svgPathMaskContainer.innerHTML = `
<defs>
  <mask id="smaller-path-${cell.id}">
    <path fill="white" d="${d}" stroke="black" stroke-width="${PATH_STROKE_WIDTH_FOR_MASK}"></path>
  </mask>
  <mask id="bigger-path-${cell.id}" >
    <path fill="white" d="${d}" stroke="black" stroke-width="${PATH_STROKE_WIDTH_FOR_MASK - PATH_STROKE_WIDTH}"></path>
  </mask>
  <mask id="main-mask-${cell.id}">
    <path fill="white" d="${d}" mask="url(#bigger-path-${cell.id})"></path>
    <path fill="black" d="${d}" mask="url(#smaller-path-${cell.id})"></path>
  </mask>
</defs>
<rect fill="black" x="0" y="0" width="${svgWidth}" height="${svgHeight}" mask="url(#main-mask-${cell.id})"></rect>
      `;
  }

  if (cell.svg.type === 'circle') {
    svgPathMaskContainer.innerHTML = `
<defs>
  <mask id="smaller-path">
    <circle fill="white" cx="${cell.svg.cx}" cy="${cell.svg.cy}" r="${
      cell.svg.r
    }" stroke="black" stroke-width="${PATH_STROKE_WIDTH_FOR_MASK}"></circle>
  </mask>
  <mask id="bigger-path" >
    <circle fill="white" cx="${cell.svg.cx}" cy="${cell.svg.cy}" r="${cell.svg.r}" stroke="black" stroke-width="${
      PATH_STROKE_WIDTH_FOR_MASK - PATH_STROKE_WIDTH
    }"></circle>
  </mask>
  <mask id="main-mask">
    <circle fill="white" cx="${cell.svg.cx}" cy="${cell.svg.cy}" r="${cell.svg.r}" mask="url(#bigger-path)"></circle>
    <circle fill="black" cx="${cell.svg.cx}" cy="${cell.svg.cy}" r="${cell.svg.r}" mask="url(#smaller-path)"></circle>
  </mask>
</defs>

<rect fill="black" x="0" y="0" width="${svgWidth}" height="${svgHeight}" mask="url(#main-mask)"></rect>
      `;
  }
  return svgPathMaskContainer;
};

export const buildImageForCanvasFromHtmlStr = (htmlStr) => {
  return new Promise((resolve) => {
    const DOMURL = window.URL || window;
    const img = new Image();
    const svgBlob = new Blob([htmlStr], { type: 'image/svg+xml' });
    const url = DOMURL.createObjectURL(svgBlob);
    img.onload = () => {
      DOMURL.revokeObjectURL(img);
      resolve(img);
    };
    img.src = url;
  });
};

export const didStarsChange = (currentGameValue, previousGameValue) => {
  return Object.keys(currentGameValue.stocks).some(
    (firmId) => currentGameValue.stocks[firmId].stars !== previousGameValue.stocks[firmId].stars
  );
};

export const shouldRedraw = (currentGameValue, previousGameValue) => {
  let isPlayersFirmsChanged;
  // check if change showWesternBackground in settings;
  // TODO...
  /*if (settings) {
    return (
      changes.settings?.currentValue?.disableEnable?.showWesternBackground !==
      changes.settings?.previousValue?.disableEnable?.showWesternBackground
    );
  }*/

  // deep compare player's firms;
  if (previousGameValue) {
    isPlayersFirmsChanged = previousGameValue.players.order.some((playerIdKey) => {
      if (!currentGameValue.players[playerIdKey]) {
        // player was droped or lost;
        return true;
      }
      return (
        currentGameValue.players[playerIdKey].firms.length !== previousGameValue.players[playerIdKey].firms.length ||
        didStarsChange(currentGameValue, previousGameValue)
      );
    });
  }

  return isPlayersFirmsChanged;
};

export const getMaxSide = (cell) => {
  const width = cell.location.x2 - cell.location.x1;
  const height = cell.location.y2 - cell.location.y1;
  return width > height ? width : height;
};
